###############################################################
#   
#  File:      core_def_spec.rb
#
#  Author:    Christian Hättich
#
#  Project:   System-On-Chip Maker
#
#  Target:    Linux / Windows / Mac
#
#  Language:  ruby
#
#
###############################################################
#
#
#   Copyright (C) 2014  Christian Hättich  - feddischson [ at ] opencores.org
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
###############################################################
#
#   Description:
#     Test specification for SOCMaker::CoreDef
#
#
#
#
###############################################################
require_relative( 'spec_helper' )

require "yaml_examples.rb"


describe SOCMaker::CoreDef do

  include_context "yaml examples"

  describe '#initialize' do

    context 'with the minimum of input arguments' do
      subject { SOCMaker::CoreDef.new(  "acore", "acore,v1", "top"  ) }
      its( :class     )         { should == SOCMaker::CoreDef }
      its( :name      )         { should ==  "acore"          }
      its( :id        )         { should == :"acore,v1"       }
      its( :toplevel  )         { should ==  "top"            }

      its( :vccmd )             { should == "" }
      its( :description )       { should == "" }
      its( :date )              { should == "" }
      its( :license )           { should == "" }
      its( :licensefile )       { should == "" }
      its( :author )            { should == "" }
      its( :authormail )        { should == "" }
      its( :vccmd )             { should == "" }
      its( :interfaces )        { should == {} }
      its( :functions )         { should == {} }
      its( :inst_parameters )   { should == {} }
      its( :static_parameters ) { should == {} }
      its( :hdlfiles )          { should == {} }
    end



    context 'from yaml strings' do
      it "not raise any error" do
        @valid_yamls.each do |setup|
          SOCMaker::from_s( setup[:yaml] ).class.should == SOCMaker::CoreDef
        end
      end
    end


    context 'with an invalid name ' do
      it "raises InitError when nil" do
        expect{ SOCMaker::CoreDef.new( nil, "v1", "top" ) }.
          to raise_error( SOCMaker::ERR::InitError )
      end

      it "raises InitError when zero-length" do
        expect{ SOCMaker::CoreDef.new( "", "v1", "top" ) }.
          to raise_error( SOCMaker::ERR::InitError )
      end

      it "raise InitError when not of type string" do
        expect{ SOCMaker::CoreDef.new( 3, "v1", "top" ) }.
          to raise_error( SOCMaker::ERR::InitError )
      end
    end




    context 'with an invalid id' do
      it "raises InitError when nil" do
        expect{ SOCMaker::CoreDef.new( "acore", nil, "top" ) }.
          to raise_error( SOCMaker::ERR::InitError )
      end

      it "raises InitError when zero-length" do
        expect{ SOCMaker::CoreDef.new( "acore", "", "top" ) }.
          to raise_error( SOCMaker::ERR::InitError )
      end

      it "raises InitError when not of type string " do
        expect{ SOCMaker::CoreDef.new( "acore", [ 1, 2, 3 ], "top" ) }.
          to raise_error( SOCMaker::ERR::InitError )
      end
    end



    context 'with an invalid toplevel name' do
      it "raises InitError when nil" do
        expect{ SOCMaker::CoreDef.new( "acore", "acore,v1", nil ) }.
          to raise_error( SOCMaker::ERR::InitError )
      end

      it "raises InitError when zero-length" do
        expect{ SOCMaker::CoreDef.new( "acore", "acore,v1", "" ) }.
          to raise_error( SOCMaker::ERR::InitError )
      end

      it "raises InitError when not of type string " do
        expect{ SOCMaker::CoreDef.new( "acore", "acore,v1", [ 1, 2, 3 ] ) }.
          to raise_error( SOCMaker::ERR::InitError )
      end
    end


    context 'auto-completion of field' do
      %w[ description date license licensefile 
          author authormail vccmd ].each do |m|
        it "#{m} to an empty string" do
          c = SOCMaker::CoreDef.new( "acore", "v1", "top" )
          c.instance_variable_get( '@'+m ).class.should be == String
          c.instance_variable_get( '@'+m ).should be       == ""
        end
      end
      %w[ interfaces functions        
          inst_parameters 
          static_parameters 
          hdlfiles ].each do |m|
        it "#{m} to an empty Hash" do
          c = SOCMaker::CoreDef.new( "acore", "v1", "top" )
          c.instance_variable_get( '@'+m ).class.should be == Hash
          c.instance_variable_get( '@'+m ).should be       == {}
        end
      end
    end


    context 'with an invalid' do
      %w[ description date license licensefile 
          author authormail vccmd ].each do |m|
        it "#{m} (not as string)" do
        # pass an numerical value
        expect{ SOCMaker::CoreDef.new( "acore", "v1", "top", { m => 4 } ) }.
          to raise_error( SOCMaker::ERR::InitError )
        end
      end

      %w[ interfaces functions        
          inst_parameters 
          static_parameters hdlfiles ].each do |m|
        it "#{m} (not as hash)" do
          # pass an numerical value
          expect{ SOCMaker::CoreDef.new( "acore", "v1", "top", { m => 4 } ) }.
            to raise_error( SOCMaker::ERR::InitError )
        end
      end
    end


    context 'with nil entry of hash' do
      %w[ interfaces
          inst_parameters 
          static_parameters
          hdlfiles ].each do |field|
        it "#{field}" do
          expect{  SOCMaker::CoreDef.new( "A core", 
                                          "acore,v1", 
                                          "top", 
                                          field => { "entry-name" => nil } )  }
              .to raise_error( SOCMaker::ERR::InitError )
        end
      end

    end

  end

  describe "#dir_name" do
    let( :inst ) { SOCMaker::CoreDef.new( "A Core", "acore,v1", "top" ) }
    it "returns an id without ','" do
      expect( inst.dir_name ).to eq( "acore_v1" )
    end
  end



  describe "#add_interface" do

    before( :each ) do

      ifc_s1 = SOCMaker::IfcSpc.new( "Interface 2", "i1,v1", 'ports' => { x1:{dir:1}, x2:{dir:0} } )
      SOCMaker::lib.add_ifc( ifc_s1 )
      @p1 = SOCMaker::IfcPort.new( "x1", 1 )
      @p2 = SOCMaker::IfcPort.new( "x2", 2 )
      @c = SOCMaker::CoreDef.new( "A core", "acore,v1", "top" )
      @c.add_interface( 'my_ifc', 'i1,v1', 0, { m_x1: @p1, m_x2: @p2 } ) 
    end
    after( :each ) do
      SOCMaker::lib.clear
    end

    context "error handling" do
      it "raises an error if name is used twice" do
        expect{ @c.add_interface( 'my_ifc', 'i1,v1', 0, { m_x1: @p1, m_x2: @p2 } ) }.to raise_error( SOCMaker::ERR::ProcessingError )
      end
    end

    it "adds an interface" do
      expect( @c.interfaces[ :my_ifc ] ).not_to be_nil
    end

  end


  describe "#to_s" do
    before( :each ) do
      @c = SOCMaker::CoreDef.new( "A core", "acore,v1", "top" )
    end

    context "string creation" do
      it{ expect( @c.to_s ).not_to be_nil }
      it{ expect( @c.to_s.size ).to be > 0 }
    end
  end


  describe "#all_static_parameters" do
    before( :each ) do
      @c = SOCMaker::CoreDef.new( "A core", "acore,v1", "top" )
    end
    context "empty hash" do
      it {expect( @c.all_static_parameters ).to eq( {} ) }
    end
  end

  describe "::get_and_ensure_dir!" do
    before( :each ) do

      @build_dir_bak =  SOCMaker::conf[ :build_dir ]
      @hdl_dir_bak   =  SOCMaker::conf[ :hdl_dir   ]
      @build_dir = "./spec/test_build_dir"
      @hdl_dir   = "./some/hdl/path/sub_dir"
      @sub_dir   = "my_dir_name"
      SOCMaker::conf[ :build_dir ] = @build_dir
      SOCMaker::conf[ :hdl_dir   ] = @hdl_dir
      @final_dir = File.expand_path( File.join( @build_dir, @hdl_dir, @sub_dir ) ) 
    end
    after( :each ) do
      SOCMaker::conf[ :build_dir ] = @build_dir_bak
      SOCMaker::conf[ :hdl_dir   ] = @hdl_dir_bak
      FileUtils.rm_rf( @build_dir )
    end

    context "directory creation" do
      it do
        expect( SOCMaker::CoreDef::get_and_ensure_dst_dir!( @sub_dir ) ).to eq(  @final_dir ) 
        expect( File.exist?( @final_dir ) ).to be( true )
        expect( SOCMaker::CoreDef::get_and_ensure_dst_dir!( @sub_dir ) ).to eq(  @final_dir ) 
      end
    end

  end




  describe "#ifc_specification" do

    after( :each ) do
      SOCMaker::lib.clear
    end

    it "should return the spec instances" do
      ifc_spc = SOCMaker::IfcSpc.new( "Ifc 1", "i1,v1", 'ports' => { p1:{dir:1} } )
      SOCMaker::lib.add_ifc( ifc_spc )
      p1 = SOCMaker::IfcPort.new( "p1", 1 )
      inst = SOCMaker::CoreDef.new( "A Core", "acore,v1", "top", "interfaces" => 
                      { "my_ifc" =>  SOCMaker::IfcDef.new( "i1", "i1,v1", 0, { m_p1: p1 } ) } )
      expect( inst.ifc_specification( "my_ifc" ) ).to equal( ifc_spc )
    end
  end


  describe "#update_vcs" do
    it "should not do any 'system' call when no vccmd is given" do
      inst = SOCMaker::CoreDef.new( "A Core", "acore,v1", "top" )
      inst.should_not_receive(:system)
      inst.update_vcs
    end

    it "should a 'system' call when a vccmd is given" do
      inst = SOCMaker::CoreDef.new( "A Core", "acore,v1", "top", "vccmd" => "svn co my-url" )
      inst.src_dir = "dir"
      inst.should_receive(:system).with( "cd dir && svn co my-url" )
      inst.update_vcs
    end
  end


  describe "::core_cnt" do
    context "hash creation" do
      it{ expect( SOCMaker::CoreDef::core_cnt( %w[ i1 i2 i3 i2 i3 i4 i5 ] ) )
          .to eq( { "i1" => 1, "i2" => 2, "i3" => 2, "i4" => 1, "i5"=> 1 } ) }
    end
  end

  describe "#all_core_id" do
    before( :each ) do
      @c = SOCMaker::CoreDef.new( "A core", "acore,v1", "top" )
    end
    context "return of id array" do
      it{ expect( @c.all_core_id ).to eq( [ :"acore,v1" ] ) }
    end

  end

  describe "#generics" do
    it 'should iterate over all generics (inst. parameters)' do


      p1 = SOCMaker::Parameter.new( "integer" )
      p2 = SOCMaker::Parameter.new( "string" )
      p3 = SOCMaker::Parameter.new( "integer" )
      p4 = SOCMaker::Parameter.new( "string" )

      c = SOCMaker::CoreDef.new( "acore", "v1", "top",
        { 'inst_parameters' => { p1: p1, p2: p2, p3: p3, p4: p4 } } )

      a_name    = [];
      a_type    = [];
      a_default = [];
      a_is_last = [];
      c.generics do |name,type,default,is_last| 
        a_name    << name
        a_type    << type
        a_default << default
        a_is_last << is_last
      end
      a_name.should be == %w[ p1 p2 p3 p4 ]
      a_type.should be == %w[ integer string integer string ]
      a_default.should be == [ 0, 0, 0, 0 ]
      a_is_last.should be == [ false, false, false, true ]
    end
  end




  describe "#ports" do

    after( :each ) do
      SOCMaker::lib.clear
    end

    before( :each ) do
      ifc_s1 = SOCMaker::IfcSpc.new( "Interface 1", "i1,v1", 'ports' => { p1:{dir:1}, p2:{dir:1}, p3:{dir:0,default:'X'} } )
      ifc_s2 = SOCMaker::IfcSpc.new( "Interface 2", "i2,v1", 'ports' => { x1:{dir:1}, x2:{dir:0} } )
      SOCMaker::lib.add_ifc( ifc_s1 )
      SOCMaker::lib.add_ifc( ifc_s2 )

      p1 = SOCMaker::IfcPort.new( "p1", 1 )
      p2 = SOCMaker::IfcPort.new( "p2", 2 )
      p3 = SOCMaker::IfcPort.new( "p3", 3 )
      x1 = SOCMaker::IfcPort.new( "x1", 1 )
      x2 = SOCMaker::IfcPort.new( "x2", 2 )

      ifc_d1 = SOCMaker::IfcDef.new( "i1", "i1,v1", 0, { m_p1: p1, m_p2: p2, m_p3: p3 } )
      ifc_d2 = SOCMaker::IfcDef.new( "i2", "i2,v1", 0, { m_x1: x1, m_x2: x2           } )

      @c = SOCMaker::CoreDef.new( "A core", "acore,v1", "top",
        { 'interfaces' =>  { i1: ifc_d1, i2: ifc_d2 } } )
    end

    it 'should iterate over all ports of all interfaces' do
      r_name    = []
      r_dir     = []
      r_len     = []
      r_default = []
      r_ref     = []
      r_is_last = []
      @c.ports do |arg_name,arg_dir,arg_len,arg_default,arg_ref,arg_is_last|
        r_name    << arg_name
        r_dir     << arg_dir
        r_len     << arg_len
        r_default << arg_default
        r_ref     << arg_ref
        r_is_last << arg_is_last
      end
      expect( r_name.sort     ).to eq( %w[ m_p1 m_p2 m_p3 m_x1 m_x2 ].sort   )
      expect( r_dir.sort      ).to eq( [ 1, 1, 0, 1, 0 ].sort                )
      expect( r_len.sort      ).to eq( [ 1, 2, 3, 1, 2 ].sort                )
      expect( r_default.sort  ).to eq( [ "0", "0", "X", "0", "0" ].sort      )
      expect( r_ref.sort      ).to eq( [ "p1", "p2", "p3", "x1", "x2" ].sort )
      expect( r_is_last   ).to eq( [ false, false, false, false, true ]    )
    end

    it 'should iterate over all ports of a specific interface' do
      r_name    = []
      r_dir     = []
      r_len     = []
      r_default = []
      r_ref     = []
      r_is_last = []
      @c.ports( "i1" ) do |arg_name, arg_dir, arg_len, arg_default, arg_ref, arg_is_last|
        r_name    << arg_name
        r_dir     << arg_dir
        r_len     << arg_len
        r_default << arg_default
        r_ref     << arg_ref
        r_is_last << arg_is_last
      end
      expect( r_name.sort ).to eq( %w[ m_p1 m_p2 m_p3 ].sort  )
      expect( r_dir.sort  ).to eq( [ 1, 1, 0  ].sort          )
      expect( r_len.sort  ).to eq( [ 1, 2, 3  ].sort          )
      expect( r_default.sort  ).to eq( [ "0", "0", "X" ].sort )
      expect( r_ref.sort      ).to eq( [ "p1", "p2", "p3" ].sort )
      expect( r_is_last   ).to eq( [ false, false, true ]     )
    end
  end






  describe "#consistency_check" do

      it "should throw an error if an incomplete interface is used" do

        # three (auto) mandatory ports
        ifc_s1 = SOCMaker::IfcSpc.new( "i1", "v1", 'ports' => { p1: { dir: 1}, p2: {dir: 1}, p3: {dir:0} } )
        SOCMaker::lib.add_ifc( ifc_s1 )

        # interface implementaiton with only two of the three ports
        p1 = SOCMaker::IfcPort.new( "p1", 1 )
        p2 = SOCMaker::IfcPort.new( "p2", 2 )
        ifc_d1 = SOCMaker::IfcDef.new( "i1", "v1", 0, { m_p1: p1, m_p2: p2 } )

        c = SOCMaker::CoreDef.new( "acore", "v1", "top",
          { 'interfaces' =>  { i1: ifc_d1 } } )


      expect{ c.consistence_check }.
        to raise_error( SOCMaker::ERR::ConsistenceError )

      end
  end

  describe "object handling, en-decoding:" do

    it "should be possible to encode and decode a core definition" do
      o1 = SOCMaker::CoreDef.new( "acore", "acore,v1", "top" )
      yaml_str = o1.to_yaml
      o2 = YAML::load( yaml_str )
      o1.should be == o2
    end

    it "should return false for two non-equal objects" do
      file = { "file.vhd".to_sym => SOCMaker::HDLFile.new( "./file.vhd" ) }
      o1 = SOCMaker::CoreDef.new( "acore", "acore,v1", "top", "hdlfiles" =>  file )
      o2 = Marshal::load(Marshal.dump(o1))
      o2.id = "acore,v2"
      ( o2 == o1 ).should be == false
      o2 = Marshal::load(Marshal.dump(o1))
      o2.hdlfiles[ "file.vhd".to_sym ].use_syn = false
      ( o2 == o1 ).should be == false
    end

  end

end




# vim: noai:ts=2:sw=2
