###############################################################
#   
#  File:      core_inst_spec.rb
#
#  Author:    Christian Hättich
#
#  Project:   System-On-Chip Maker
#
#  Target:    Linux / Windows / Mac
#
#  Language:  ruby
#
#
###############################################################
#
#
#   Copyright (C) 2014  Christian Hättich  - feddischson [ at ] opencores.org
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
###############################################################
#
#   Description:
#
#
#
#
###############################################################
require_relative( 'spec_helper' )


describe SOCMaker::CoreInst do


  before( :each ) do

    p1 = SOCMaker::IfcPort.new( "p1", 12 )
    @core = SOCMaker::CoreDef.new( "mycore", "mycore,rel1", "core_top",
        "interfaces"      => { "my_ifc" =>  SOCMaker::IfcDef.new( "i1", "i1,v1", 0, { m_p1: p1 } ) },
        "inst_parameters" => { "param1".to_sym => SOCMaker::Parameter.new( "integer" )             } )


    ifc_spc = SOCMaker::IfcSpc.new( "Ifc 1", "i1,v1", 'ports' => { p1:{dir:1} } )
    SOCMaker::lib.add_ifc( ifc_spc )
    SOCMaker::lib.add_core( @core )
  end

  after( :each ) do
    SOCMaker::lib.clear
  end


  describe "#initialize" do

    it "raises an error, if the type is nil" do 
       # add with nil type
       expect{  SOCMaker::CoreInst.new( nil, { "aparameter".to_sym => 4 } ) } .
       to raise_error( SOCMaker::ERR::InitError )
    end

    context "creates an instance" do
       # add with mycore,rel1 type
       subject{ SOCMaker::CoreInst.new( "mycore,rel1", { "aparameter".to_sym => 4 } ) }
       its( :class ) { should == SOCMaker::CoreInst }
       its( :type  ) { should == :"mycore,rel1" }
    end

    it "raises an error, if parameters are not given as hash" do
       expect{  SOCMaker::CoreInst.new( "mycorerel1", "not a hash"  ) }.
       to raise_error( SOCMaker::ERR::InitError )
    end


    it "raises an error, if parameters are given, which doesn't exist in the definition" do
       expect{ inst = SOCMaker::CoreInst.new( "mycore,rel1", { "aparameter".to_sym => 4 } )
               inst.consistence_check }.
        to raise_error( SOCMaker::ERR::ConsistenceError )
    end

    it "auto-completes generics with default values" do

       inst = SOCMaker::CoreInst.new( "mycore,rel1", {}  )
       inst.consistence_check
       expect( inst.params[ :param1 ] ).to be == 0
    end
  end

  describe "object handling, en-decoding:" do

    it "should be possible to encode and decode a core instance" do
      o1 = SOCMaker::CoreInst.new( "mycore,rel1", {}  )
      yaml_str = o1.to_yaml
      o2 = YAML::load( yaml_str )
      expect( o1 ).to eq( o2 )
    end

    it "should return false for two non-equal objects" do
      o1 = SOCMaker::CoreInst.new( "mycore,rel1" )
      o1.consistence_check
      o2 = Marshal::load(Marshal.dump(o1))
      o2.type = ( o2.type.to_s + "X" ).to_sym
      expect( o2 == o1 ).to eq( false )
      o2 = Marshal::load(Marshal.dump(o1))
      o2.params[ :param1 ] = 1
      expect( o2 == o1 ).to eq( false )
    end
  end



  describe "#consistence_check" do
    it "raises an error, if a core def doesn't exist" do
      expect{ SOCMaker::CoreInst.new( "mycore,rel2", {} ).consistence_check }
        .to raise_error( SOCMaker::ERR::ConsistenceError ) 
    end
  end


  describe "#core_definition" do
    before( :each ) do
      @soc = SOCMaker::SOCDef.new( "mysoc", "mysoc,v1", "soc_top" )
      SOCMaker::lib.add_core( @soc )
      @inst = SOCMaker::CoreInst.new( "mycore,rel1", {}  )
      @inst.consistence_check
    end
  end



  describe "#port_length" do
    before( :each ) do
      p1 = SOCMaker::IfcPort.new( "p1", 21 )

      @inst1 = SOCMaker::CoreInst.new( "mycore,rel1", {}  )
      @soc = SOCMaker::SOCDef.new( "mysoc", "mysoc,v1", "soc_top",
        "interfaces"      => { "my_ifc" =>  SOCMaker::IfcDef.new( "i1", "i1,v1", 0, { m_p1: p1 } ) }, 
        "cores"           => { core_inst: @inst1 } )

      SOCMaker::lib.add_core( @soc )

      @inst2 = SOCMaker::CoreInst.new( "mysoc,v1", {}  )
      @inst2.consistence_check
    end

    it "returns the correct port length" do
      expect( @inst1.port_length( "my_ifc", :p1 ) ).to be( 12 )
    end

  end



end



# vim: noai:ts=2:sw=2
