###############################################################
#   
#  File:      hd_parser_spec.rb
#
#  Author:    Christian Hättich
#
#  Project:   System-On-Chip Maker
#
#  Target:    Linux / Windows / Mac
#
#  Language:  ruby
#
#
###############################################################
#
#
#   Copyright (C) 2015  Christian Hättich  - feddischson [ at ] opencores.org
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
###############################################################
#
#   Description:
#     Test specification for SOCMaker::HDLFile
#
#
#
###############################################################
require_relative( 'spec_helper' )




describe SOCMaker::HDLParser do

  before( :each ){ @inst = SOCMaker::HDLParser.instance }


  describe "#get_extension" do
    it "returns '.vhd' for a vhdl file" do
      expect( @inst.get_extension( "./some/file.vhd" ) ).to eq( ".vhd")
    end
    it "returns '.v' for a verilog file" do
      expect( @inst.get_extension( "./some/file.v" ) ).to eq( ".v")
    end
  end


  describe "#parse_core" do
    it "should return a string" do
      result = SOCMaker::HDLParser.instance.parse_core( 
          "spec/test_soc_lib/cores/core_A_rel1/core_a.vhd",
          "spec/test_soc_lib/cores/core_A_rel1/*.vhd"  )  
      expect( result ).to be_a( String )
    end
  end


end


describe SOCMaker::VHDLParser do

  before( :each ){ 
    @inst = SOCMaker::VHDLParser.instance 
    @example = "
 entity mux is
    generic(
        w1 : natural := 4;
        w2 : natural := 4;
        wx : std_logic_vector( 123-1 downto 0 )
        );
    port (
        din_0   :in  std_logic_vector( 4-1 downto 0 ); -- first input
        din_1   :in  std_logic_vector( w1-1 downto 0 ); -- second input
        sel     :in  std_logic;-- Select input 1
        mux_out :out std_logic_vector( 4-1-3+2+1 downto 0 ); -- Mux output
        sel2    :    std_logic;                   -- Select input 2 (unused at the moment)
        sel3    :    std_logic_vector( 3 to 5 )   -- Select input 3 (unused at the moment)
    );
  end entity;

architecture behavior of mux is

begin
    with (sel) select
    mux_out <= din_0 when '0',
               din_1 when others;
end architecture; "

  }

  describe "#parse_toplevel" do

    it "returns extracts all necessary information" do

      result = @inst.parse_toplevel( @example )
      expect( result ).to eq(
          {  :name    =>"mux", 
             :generic => { 
      "w1"     => { :type=>"natural",          :name=>"w1", :default=>"4"},
      "w2"     => { :type=>"natural",          :name=>"w2", :default=>"4"}, 
      "wx"     => { :type=>"std_logic_vector", :name=>"wx", :range=>" 123-1 downto 0 "}
      }, 
             :port    => 
  { "din_0"  => { :name=>"din_0",   :type=>"std_logic_vector", :direction=>"in",  :range=>" 4-1 downto 0 "}, 
    "din_1"  => { :name=>"din_1",   :type=>"std_logic_vector", :direction=>"in",  :range=>" w1-1 downto 0 "}, 
    "sel"    => { :name=>"sel",     :type=>"std_logic",        :direction=>"in"  }, 
    "sel2"   => { :name=>"sel2",    :type=>"std_logic",        :direction=>"in"  }, 
    "mux_out"=> { :name=>"mux_out", :type=>"std_logic_vector", :direction=>"out", :range=>" 4-1-3+2+1 downto 0 "},
   "sel3"    => { :name=>"sel3",    :type=>"std_logic_vector", :direction=>"in",  :range=>" 3 to 5 "}  } } )

    end

    it "should extract the length information" do

      result = @inst.parse_toplevel( @example )
      result = @inst.extract_length( result )

      expect( result[ :port][ "din_0" ][ :length ]  ).to equal( 4 )
      expect( result[ :port][ "din_1" ][ :length ]  ).to eq( "w1" )
      expect( result[ :port][ "sel" ][ :length ]  ).to equal( 1 )
      expect( result[ :port][ "sel2" ][ :length ]  ).to equal( 1 )
      expect( result[ :port][ "sel3" ][ :length ]  ).to equal( 3 )
      expect( result[ :port][ "mux_out" ]  [ :length ]  ).to equal( 4 )

    end

  end

end

describe SOCMaker::VerilogParser do

  before( :each ){ 
    @inst = SOCMaker::VerilogParser.instance 
    @example = "

module  mux( din_0,  din_1,  sel, mux_out );
 parameter w1 = 4;
 parameter w2 = 4;
 input   [4-1 : 0]   din_0; 
 input   [w1-1 : 0]    din_1;
 input                sel;
 input                sel2;
 input   [4:3]        sel3;
 output  [4-1:0]      mux_out;

 wire                 mux_out;
 assign mux_out = (sel) ? din_1 : din_0;
 endmodule "
  }

    it "returns extracts all necessary information" do

      result = @inst.parse_toplevel( @example )
      expect( result ).to eq(
      {   :name    => "mux", 
          :generic => {  
      "w1"  => { :name    =>"w1", :default =>"4", :type=>"integer"}, 
      "w2"  => { :name    =>"w2", :default =>"4", :type=>"integer"} }, 
          :port=>{ "din_0"   => { :name=>"din_0", :dir=>"in", :range=>"4-1 : 0" }, 
                   "din_1"   => { :name=>"din_1", :dir=>"in", :range=>"w1-1 : 0"  }, 
                   "sel"     => { :name=>"sel",  :dir=>"in"}, 
                   "sel2"    => { :name=>"sel2", :dir=>"in"}, 
                   "sel3"    => { :name=>"sel3", :dir=>"in", :range=>"4:3"}, 
                   "mux_out" => { :name=>"mux_out", :dir=>"out", :range=>"4-1:0"} } } )
    end




    it "should extract the length information" do

      result = @inst.parse_toplevel( @example )
      result = @inst.extract_length( result )


      expect( result[ :port][ "din_0" ][ :length ]  ).to equal( 4 )
      expect( result[ :port][ "din_1" ][ :length ]  ).to eq( "w1" )
      expect( result[ :port][ "sel" ][ :length ]  ).to equal( 1 )
      expect( result[ :port][ "sel2" ][ :length ]  ).to equal( 1 )
      expect( result[ :port][ "sel3" ][ :length ]  ).to equal( 2 )
      expect( result[ :port][ "mux_out" ]  [ :length ]  ).to equal( 4 )

    end


end
