###############################################################
#   
#  File:      ifc_spc_spec.rb
#
#  Author:    Christian Hättich
#
#  Project:   System-On-Chip Maker
#
#  Target:    Linux / Windows / Mac
#
#  Language:  ruby
#
#
###############################################################
#
#
#   Copyright (C) 2014  Christian Hättich  - feddischson [ at ] opencores.org
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
###############################################################
#
#   Description:
#     Test specification for SOCMaker::IfcDef
#
#
#
#
###############################################################
require_relative( 'spec_helper' )

describe SOCMaker::IfcDef do
  before( :each ){ @tmp_port = SOCMaker::IfcPort.new( "abc" ) }
  describe "#initialize" do


    it "returns a SOCMaker::IfcDef object when creating with new" do
      s = SOCMaker::IfcDef.new( "myifc", "myifc,v1", 1, { test: @tmp_port  } )
      expect( s.class ).to be( SOCMaker::IfcDef )
    end

    it "raises an error if the name is not a string" do
       expect{ SOCMaker::IfcDef.new( 4, "myifc,v1", 1, { test: @tmp_port  } )  }.
          to raise_error( SOCMaker::ERR::InitError )
    end

    it "raises an error if the direction is neither 0 nor 1 " do
       expect{ SOCMaker::IfcDef.new( "myifc", "myifc,v1", 4, { test: @tmp_port  } )  }.
          to raise_error( SOCMaker::ERR::InitError )
    end


    it "raises an error if no ports are given " do
       expect{ SOCMaker::IfcDef.new( "myifc", "myifc,v1", 4, { } )  }.
          to raise_error( SOCMaker::ERR::InitError )
    end


    it "raises an error if a ports is nil " do
       expect{ SOCMaker::IfcDef.new( "myifc", "myifc,v1", 4, { a_port: nil} )  }.
          to raise_error( SOCMaker::ERR::InitError )
    end

    it "raises an error if a ports is not of type SOCMaker::IfcPort " do
       expect{ SOCMaker::IfcDef.new( "myifc", "myifc,v1", 4, { a_port: "string-type"} )  }.
          to raise_error( SOCMaker::ERR::InitError )
    end

    it "raises an error if no ports are given (nil)" do
       expect{ SOCMaker::IfcDef.new( "myifc", "myifc,v1", 4, nil )  }.
          to raise_error( SOCMaker::ERR::InitError )
    end
  end

  describe "object handling, en-decoding" do

    it "returns false for two non-equal objects" do
      o1 = SOCMaker::IfcDef.new( "myifc", "myifc,v1", 1, { test: @tmp_port  } )
      o2 = Marshal::load(Marshal.dump(o1))
      o2.id = "myifc,v2"
      expect( o2 == o1 ).to be( false )
    end

    it "is possible to encode and decode a interface definition" do
      o1 = SOCMaker::IfcDef.new( "myifc", "myifc,v1", 1, { test: @tmp_port  } )
      yaml_str = o1.to_yaml
      o2 = YAML::load( yaml_str )
      expect( o1 ).to eq( o2 )
    end
  end

  describe  "#consistence_check" do
    it "raise an error if a port reference exists multiple times" do
      p1 = SOCMaker::IfcPort.new( "p1" )
      p2 = SOCMaker::IfcPort.new( "p2" )
      p3 = SOCMaker::IfcPort.new( "p3" )
      p4 = SOCMaker::IfcPort.new( "p3" )

      o1 = SOCMaker::IfcDef.new( "myifc", "myifc,v1", 1, 
          { t1: p1, t2: p2, t3: p3, t4: p4 } )
      expect{ o1.consistence_check }.to raise_error( SOCMaker::ERR::ConsistenceError ) 
    end
  end

end


# vim: noai:ts=2:sw=2

