###############################################################
#   
#  File:      ifc_spc_spec.rb
#
#  Author:    Christian Hättich
#
#  Project:   System-On-Chip Maker
#
#  Target:    Linux / Windows / Mac
#
#  Language:  ruby
#
#
###############################################################
#
#
#   Copyright (C) 2014  Christian Hättich  - feddischson [ at ] opencores.org
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
###############################################################
#
#   Description:
#     Test specification for SOCMaker::IfcSpc
#
#
#
#
###############################################################
require_relative( 'spec_helper' )

require "yaml_examples.rb"



describe SOCMaker::IfcSpc do
  include_context "yaml examples"

  describe "#initialize" do

    it "returns a SOCMaker::IfcSpc object when creating with new" do
      s = SOCMaker::IfcSpc.new( "My interface", "myifc,v1" )
      expect( s.class ).to be( SOCMaker::IfcSpc )
    end

    it "raises an error if the name is not a string" do
       expect{  SOCMaker::IfcSpc.new( 1234, "myifc,v1" ) }.
          to raise_error( SOCMaker::ERR::InitError )
    end

    it "raises an error if the name is an empty string" do
       expect{  SOCMaker::IfcSpc.new( "", "myifc,v1"  ) }.
          to raise_error( SOCMaker::ERR::InitError )
    end

    it "raises an error if the id is not a string" do
       expect{  SOCMaker::IfcSpc.new( "myifc", 234  ) }.
          to raise_error( SOCMaker::ERR::InitError )
    end

    it "raises an error if the id is an a empty string" do
       expect{  SOCMaker::IfcSpc.new( "myifc", ""  ) }.
          to raise_error( SOCMaker::ERR::InitError )
    end

    it "raises an error if the multiplicity has != 2 entries" do
        expect{ 
        SOCMaker::IfcSpc.new( "my ifc", "my_ifc,v1", 
                                  'ports'         => { :p1 => { dir: 1 }  },
                                  'multiplicity'  => [ 1, 0, 1 ]       ) 
        }.to raise_error( SOCMaker::ERR::InitError )
    end

    it "raises an error if the multiplicity is not '*' nor >= 0" do
        expect{ 
        SOCMaker::IfcSpc.new( "my ifc", "my_ifc,v1", 
                                  'ports'         => { :p1 => { dir: 1 }  },
                                  'multiplicity'  => [ -1, 0  ]       ) 
        }.to raise_error( SOCMaker::ERR::InitError )

        expect{ 
        SOCMaker::IfcSpc.new( "my ifc", "my_ifc,v1", 
                                  'ports'         => { :p1 => { dir: 1 }  },
                                  'multiplicity'  => [ 0, -3  ]       ) 
        }.to raise_error( SOCMaker::ERR::InitError )

        expect{ 
        SOCMaker::IfcSpc.new( "my ifc", "my_ifc,v1", 
                                  'ports'         => { :p1 => { dir: 1 }  },
                                  'multiplicity'  => [ 4, 'a'  ]       ) 
        }.to raise_error( SOCMaker::ERR::InitError )

        expect{ 
        SOCMaker::IfcSpc.new( "my ifc", "my_ifc,v1", 
                                  'ports'         => { :p1 => { dir: 1 }  },
                                  'multiplicity'  => [ 5, 'abc'  ]       ) 
        }.to raise_error( SOCMaker::ERR::InitError )
    end
  end

  describe "SOCMaker::from_s" do

    it "loads from yaml" do
       c = SOCMaker::from_s( @IFC_YAML_VALID )
       c.class.should be == SOCMaker::IfcSpc
    end


    it "raises an error if a port direction is neither 0 nor 1" do
       expect{  SOCMaker::from_s( @IFC_YAML_INVALID ) }.
          to raise_error( SOCMaker::ERR::InitError )
    end

    it "raises an error if a port direction is nil" do
       expect{  SOCMaker::from_s( @IFC_YAML_INVALID2 ) }.
          to raise_error( SOCMaker::ERR::InitError )
    end
  end


  describe "#ok?" do
    it "provides correct multiplicity info" do

      ifc = SOCMaker::IfcSpc.new( "my ifc", "my_ifc,v1", 
                                    'ports'         => { :p1 => { dir: 1 }  },
                                    'multiplicity'  => [ 5, '*'  ]       ) 

      expect( ifc.n_connections_ok?( 0, 4     ) ).to be ( true  )
      expect( ifc.n_connections_ok?( 0, 5     ) ).to be ( true  ) 
      expect( ifc.n_connections_ok?( 0, 6     ) ).to be ( false )
      expect( ifc.n_connections_ok?( 1, 0     ) ).to be ( true  ) 
      expect( ifc.n_connections_ok?( 1, 10000 ) ).to be ( true  )

    end
  end


  describe "object handling" do
    it "compares two objects" do

      ifc = SOCMaker::IfcSpc.new( "my ifc", "my_ifc,v1", 
                                    'ports'         => { :p1 => { dir: 1 }  },
                                    'multiplicity'  => [ 5, '*'  ]       ) 
      yaml_str = SOCMaker::YPP.from_yaml( ifc.to_yaml )
      ifc2 = SOCMaker::from_s( yaml_str )
      ifc2.should be == ifc 
      ifc.should be == ifc2 

      ifc.name << "X"
      ifc.should_not be == ifc2 
      ifc.name = ifc2.name

      ifc.id = :some_id
      ifc.should_not be == ifc2 
      ifc.id = ifc2.id


      ifc.multiplicity[ 0 ] = 1234
      ifc.should_not be == ifc2 
      ifc.multiplicity[ 0 ] = ifc2.multiplicity[ 0 ]


      ifc.ports[ :px ] = { dir: 1 }
      ifc.should_not be == ifc2 
      ifc.id = ifc2.id

    end

    it "creates a yaml output" do

      ifc = SOCMaker::IfcSpc.new( "my ifc", "my_ifc,v1", 
                                    'ports'         => { :p1 => { dir: 1 }  },
                                    'multiplicity'  => [ 5, '*'  ]       ) 
      yaml_str = SOCMaker::YPP.from_yaml( ifc.to_yaml )
      ifc2 = SOCMaker::from_s( yaml_str )
      expect( yaml_str.size ).to be > 0 
      ifc2 = SOCMaker::from_s( yaml_str )
      expect( ifc2 ).to eq( ifc  )
    end
  end
end


# vim: noai:ts=2:sw=2
