###############################################################
#   
#  File:      soc_lib_spec.rb
#
#  Author:    Christian Hättich
#
#  Project:   System-On-Chip Maker
#
#  Target:    Linux / Windows / Mac
#
#  Language:  ruby
#
#
###############################################################
#
#
#   Copyright (C) 2014  Christian Hättich  - feddischson [ at ] opencores.org
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
###############################################################
#
#   Description:
#
#
#
#
###############################################################
require_relative( 'spec_helper' )


describe SOCMaker::Lib do

  before( :each )do
     @lib = SOCMaker::Lib.new
  end

  describe "#initialize"  do
    subject { SOCMaker::Lib.new }
    its( :class ) { should == SOCMaker::Lib }
  end




  describe "#refresh" do

    context "path loading" do
      it "calls process_include for each path given as argument" do
        paths_res = []
        @lib.stub( :process_include ) do |arg|
           paths_res << arg 
        end
        paths = [ "first_path", "second_path" ]
        @lib.refresh( paths )
        expect( paths_res ).to eq( paths )
      end

      it "cals process_include for each path from config, if no argument is given" do
        paths_res = []
        @lib.stub( :process_include ) do |arg|
           paths_res << arg 
        end
        paths = [ "first_path", "second_path" ]
        SOCMaker::conf[ :cores_search_path ] = paths
        @lib.refresh( )
        expect( paths_res ).to eq( paths )
      end
    end

    context "yaml include loading" do
      it "returns two objects" do

        obs = []
        @lib.stub( :get_all_yaml_in_str ) do |arg|
           "SOCM_INCLUDE\ndirs:\n- folder_a\n- folder_b\n- folder_c\nSOCM_INCLUDE\ndirs:\n- folder_d\n- folder_e\n- folder_f\n" 
        end
        @lib.stub( :add_include ) do |arg|
           obs << arg
        end
        @lib.refresh( [ "some_path" ] )
        expect( obs ).to eq ( [ SOCMaker::LibInc.new( 'dirs' => ["folder_a", "folder_b", "folder_c"] ), 
                                SOCMaker::LibInc.new( 'dirs' => ["folder_d", "folder_e", "folder_f"] ) ] )
      end
    end
  end

  describe "#process_include" do

    it "raises an ProcessingError if a folder is included twice" do
      expect do
        @lib.process_include( "./empty_soc_lib_folder" ) 
        @lib.process_include( "./empty_soc_lib_folder" ) 
      end.
      to raise_error( SOCMaker::ERR::InitError )
    end
  end

  describe "#add_core/#rm_core/#add_ifc/#rm_ifc" do

    it "should be possible to add, get and remove a core" do
      c = SOCMaker::CoreDef.new( "A core", "acore,v1", "top" )
      @lib.add_core( c )
      @lib.get_core( :"acore,v1" ).should be == c
      @lib.rm_core( c )
      @lib.add_core( c )
      @lib.rm_core( c.id )
      expect { @lib.get_core( :"acore,v1" ) }.
        to raise_error( SOCMaker::ERR::ProcessingError ) 
      expect { @lib.rm_core( 123 ) }.
        to raise_error( SOCMaker::ERR::ProcessingError ) 
    end

    it "should be possible to add, get and remove an interface" do
      i = SOCMaker::IfcSpc.new( "My Interface", "myifc,v2" )

      # removing with instance
      @lib.add_ifc( i )
      @lib.get_ifc( :"myifc,v2" ).should be == i
      @lib.rm_ifc( i )

      expect { @lib.get_ifc( :"myifc,v2" ) }. 
        to raise_error( SOCMaker::ERR::ProcessingError ) 

      # removing with id
      @lib.add_ifc( i )
      @lib.get_ifc( :"myifc,v2" ).should be == i
      @lib.rm_ifc( i.id )

      expect { @lib.get_ifc( :"myifc,v2" ) }
        .to raise_error( SOCMaker::ERR::ProcessingError ) 

      expect { @lib.rm_ifc( 4 ) }
        .to raise_error( SOCMaker::ERR::ProcessingError )

    end
  end

  describe  "#add_include" do
    it "should process all folders in add_include" do
      all_folders = ["folder_a", "folder_b", "folder_c" ]
      i = SOCMaker::LibInc.new( 'dirs' => all_folders  )
      all_folders_res = []
      @lib.stub( :process_include ) do |arg|
        all_folders_res << arg
      end
      @lib.send( :add_include, i, "./" )
      all_folders.each_with_index do |f,index|
        File.expand_path( File.join( "./", f ) ).should be == all_folders_res[ index ]
      end
    end
  end


  describe "#refresh" do
    it "should load all elements from our test library" do
      @lib.refresh( './spec/test_soc_lib' )
      core_A      = @lib.get_core( :"core_A,rel1"  )
      core_B      = @lib.get_core( :"core_B,rel1"  )
      core_AB_ifc = @lib.get_ifc( :"core_AB_ifc,1" )
      expect( core_A.class                  ).to be( SOCMaker::CoreDef )
      expect( core_B.class                  ).to be( SOCMaker::CoreDef )
      expect( core_AB_ifc.class             ).to be( SOCMaker::IfcSpc  )
      expect( core_A.static_parameters.size ).to be( 3 )
    end
  end


  describe "#to_s" do

    context "string creation" do
      it{ expect( @lib.to_s ).not_to be_nil }
      it{ expect( @lib.to_s.size ).to be > 0 }
    end
  end

  describe "#cores" do

    it "should be possible to add, get and remove a core" do
      c1 = SOCMaker::CoreDef.new( "A core", "acore,v1", "top" )
      c2 = SOCMaker::CoreDef.new( "A core", "acore,v2", "top" )
      c3 = SOCMaker::CoreDef.new( "A core", "acore,v3", "top" )

      @lib.add_core( c1 )
      @lib.add_core( c2 )
      @lib.add_core( c3 )
      cores = {}
      @lib.cores { |k,v| cores[k] = v }
      expect( cores ).to eq( "acore,v1" => c1, 
                             "acore,v2" => c2,
                             "acore,v3" => c3 )
    end
  end
end

# vim: noai:ts=2:sw=2

